/**
 * \file
 * ${moduleGroupNameLowerCase}_h
 *
 * \defgroup
 * i2c_client_interface
 *
 * \brief
 * This file contains API prototypes and other data types for ${moduleGroupNameLowerCase} driver.
 *
 * \version
 * I2C Client Driver Version ${clientDriverVersion}
 */

${disclaimer}

#ifndef ${moduleGroupNameUpperCase}_CLIENT_INTERFACE_H
#define ${moduleGroupNameUpperCase}_CLIENT_INTERFACE_H
/**
  Section: Included Files
*/
#include <stdbool.h>
#include <stdint.h>
#include "${moduleGroupNameLowerCase}_client_types.h"
    
/**
 Section: Data Type Definitions
 */

/**
 * \defgroup ${moduleGroupNameLowerCase}_client_interface
 * @strcut ${moduleGroupNameLowerCase}_client_interface_t
 * @brief Structure containing the function pointers of ${moduleGroupNameUpperCase} driver.
 */
struct ${moduleGroupNameUpperCase}_CLIENT_INTERFACE
{
    void (*Initialize)(void);
    void (*Deinitialize)(void);
    void (*WriteByte)(uint8_t wrByte);
    uint8_t(*ReadByte)(void);
    ${moduleGroupNameLowerCase}_client_error_t (*ErrorGet)(void);
    ${moduleGroupNameLowerCase}_client_transfer_dir_t (*TransferDirGet)(void);
    ${moduleGroupNameLowerCase}_client_ack_status_t (*LastByteAckStatusGet)(void);
    void (*CallbackRegister)(bool (*handler)(${moduleGroupNameLowerCase}_client_transfer_event_t clientEvent));
    void (*Tasks)(void);
};

/**
 * @ingroup ${moduleGroupNameLowerCase}_client
 * @example ${moduleGroupNameLowerCase}_client_interface_example_code
 * @code
 * #define I2C_EEPROM_SIZE 10
 *
 * //Private functions
 *
 * static bool EEPROM_Application(${moduleGroupNameLowerCase}_client_transfer_event_t event);
 *  
 * // Private variable
 *
 * volatile static uint8_t CLIENT_EEPROM[I2C_EEPROM_SIZE] = {
 *     0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09
 * };
 * 
 * static uint8_t memoryLocation = 0x00;
 * static bool isMemoryLocation = false;
 * 
 * // Main application
 * 
 * int main(void)
 * {
 *     SYSTEM_Initialize();
 *     ${componentNameLowerCase}_client_interface.CallbackRegister(EEPROM_Application);
 *     while (1)
 *     {
<#if !isInterruptDriven>
 *         ${componentNameLowerCase}_client_interface.Tasks();
</#if>
 *     }
 * }
 * 
 * static bool EEPROM_Application(${moduleGroupNameLowerCase}_client_transfer_event_t event)
 * {
 *     switch (event)
 *     {
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_ADDR_MATCH:
 *         if (${componentNameLowerCase}_client_interface.TransferDirGet() == ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_DIR_WRITE)
 *         {
 *             isMemoryLocation = true;
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_RX_READY:
 *         //Read the data sent by I2C Host
 *         if (isMemoryLocation)
 *         {
 *             memoryLocation = ${componentNameLowerCase}_client_interface.ReadByte();
 *             isMemoryLocation = false;
 *             break;
 *         }
 *         else
 *         {
 *             CLIENT_EEPROM[memoryLocation++] = ${componentNameLowerCase}_client_interface.ReadByte();
 *             if (memoryLocation >= I2C_EEPROM_SIZE)
 *             {
 *                 memoryLocation = 0x00;
 *             }
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_TX_READY:
 *         // Provide the EEPROM data requested by the I2C Host
 *         ${componentNameLowerCase}_client_interface.WriteByte(CLIENT_EEPROM[memoryLocation++]);
 *         if (memoryLocation >= I2C_EEPROM_SIZE)
 *         {
 *             memoryLocation = 0x00;
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_STOP_BIT_RECEIVED:
 *         memoryLocation = 0x00;
 *         break;
 *
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_ERROR:
 *         memoryLocation = 0x00;
 *         ${moduleGroupNameLowerCase}_client_error_t errorState = ${componentNameLowerCase}_client_interface.ErrorGet();
 *         if(errorState == ${moduleGroupNameUpperCase}_CLIENT_ERROR_BUS_ERROR)
 *         {
 *             // Bus Collision Error Handling
 *         }
 *         else if(errorState == ${moduleGroupNameUpperCase}_CLIENT_ERROR_COLLISION)
 *         {
 *             // Write Collision Error Handling
 *         }
 *         break;
 * 
 *     default:
 *         break;
 *     }
 *     return true;
 * }
 * @endcode
*/

#endif // end of ${moduleGroupNameUpperCase}_CLIENT_INTERFACE_H