/**
  @Company
    Microchip Technology Inc.

  @Description
    This Source file provides APIs.
    Generation Information :
    Driver Version    :   1.0.0
*/
${disclaimer}

#ifndef ${moduleNameUpperCase}_CLIENT_INTERRUPT_H
#define ${moduleNameUpperCase}_CLIENT_INTERRUPT_H

#include <stdbool.h>
#include <stdint.h>
#include "../system/utils/compiler.h"
#include "${moduleGroupNameLowerCase}_client_types.h"
#include "${moduleGroupNameLowerCase}_client_interface.h"

/**
 Section: Data Type Definitions
*/

/**
 * @ingroup ${moduleGroupNameLowerCase}_client
 * @brief External object for ${moduleGroupNameLowerCase}_client_interface
 */
extern const struct ${moduleGroupNameUpperCase}_CLIENT_INTERFACE ${componentNameLowerCase}_client_interface;

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This routine initializes the ${moduleNameUpperCase} module.
 *        This routine must be called before any other ${moduleNameUpperCase} routine is called.
 *        This routine should only be called once during system initialization.
 * @param void
 * @return void
 * @ref ${componentNameLowerCase}_client_example_code
 */
void ${componentName}_Initialize(void);

/**
 * @ingroup ${componentNameUpperCase}_host
 * @brief This API Deinitializes the ${componentName} driver.
 *        This routine disables the ${componentName} module.
 * @param void
 * @return void
 * @ref ${componentNameLowerCase}_host_example_code
 */
void ${componentName}_Deinitialize(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function writes data to a host on the bus.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param [in] wrByte - ${moduleNameUpperCase} client write byte
 * @return void
 * @ref ${componentNameLowerCase}_client_example_code
 */
void ${componentName}_WriteByte(uint8_t wrByte);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function read data from a host on the bus.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void
 * @return data byte -${moduleNameUpperCase} client read byte
 * @ref ${componentNameLowerCase}_client_example_code
 */
uint8_t ${componentName}_ReadByte(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function read requested address from a host on the bus.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void
 * @return address byte -${moduleNameUpperCase} client read byte
 * @ref ${componentNameLowerCase}_client_example_code
 */
uint16_t ${componentName}_ReadAddr(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function the error occurred during ${moduleGroupNameUpperCase} Transmit and Receive.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void
 * @return
 *        ${moduleGroupNameUpperCase}_CLIENT_ERROR_BUS_COLLISION    - ${moduleGroupNameUpperCase} Bus Collision Error
 *        ${moduleGroupNameUpperCase}_CLIENT_ERROR_WRITE_COLLISION  - ${moduleGroupNameUpperCase} Write Collision Error
 *        ${moduleGroupNameUpperCase}_CLIENT_ERROR_RECEIVE_OVERFLOW - ${moduleGroupNameUpperCase} Receive overflow
 *        ${moduleGroupNameUpperCase}_CLIENT_ERROR_NONE             - No Error
 * @ref ${componentNameLowerCase}_client_example_code
 */
${moduleGroupNameLowerCase}_client_error_t ${componentName}_ErrorGet(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function get the ${moduleGroupNameUpperCase} Transfer direction.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void
 * @return
 *        ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_DIR_WRITE  - ${moduleGroupNameUpperCase} Host is writing to client
 *        ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_DIR_READ   - ${moduleGroupNameUpperCase} Host is reading from client
 * @ref ${componentNameLowerCase}_client_example_code
 */
${moduleGroupNameLowerCase}_client_transfer_dir_t ${componentName}_TransferDirGet(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function get the ${moduleGroupNameUpperCase} Host ACK status.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void
 * @return
 *        ${moduleGroupNameUpperCase}_CLIENT_ACK_STATUS_RECEIVED_ACK   - ${moduleGroupNameUpperCase} Host is send ACK to client
 *        ${moduleGroupNameUpperCase}_CLIENT_ACK_STATUS_RECEIVED_NACK  - ${moduleGroupNameUpperCase} Host is send NACK to client
 * @ref ${componentNameLowerCase}_client_example_code
 */
${moduleGroupNameLowerCase}_client_ack_status_t ${componentName}_LastByteAckStatusGet(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief This function sets the pointer to be called "back" when
 *        the given ${moduleGroupNameUpperCase} transfer events, Bus collision and Write collision occur.
 * @param handler - A pointer to a function.
 * @return void
 * @ref ${componentNameLowerCase}_client_example_code
 */
void ${componentName}_CallbackRegister(bool (*callback)(${moduleGroupNameLowerCase}_client_transfer_event_t clientEvent));

/**
 * @ingroup ${componentNameLowerCase}_client
 * @brief his function is common Polling function for all ${moduleNameUpperCase} client.
 *        ${componentName}_Initialize() must have been called for the associated.
 * @param void.
 * @return true - Module is enabled
 *         false - Module is disabled
 * @ref ${componentNameLowerCase}_client_example_code
 */
void ${componentName}_Tasks(void);

/**
 * @ingroup ${componentNameLowerCase}_client
 * @example ${componentNameLowerCase}_client_example_code
 * @code
 * #define I2C_EEPROM_SIZE 10
 *
 * //Private functions
 *
 * static bool EEPROM_Application(${moduleGroupNameLowerCase}_client_transfer_event_t event);
 *  
 * // Private variable
 *
 * volatile static uint8_t CLIENT_EEPROM[I2C_EEPROM_SIZE] = {
 *     0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09
 * };
 * 
 * static uint8_t memoryLocation = 0x00;
 * static bool isMemoryLocation = false;
 * 
 * // Main application
 * 
 * int main(void)
 * {
 *     SYSTEM_Initialize();
 *     ${componentName}_CallbackRegister(EEPROM_Application);
 *     while (1)
 *     {
 *          ${componentName}_Tasks();
 *     }
 * }
 * 
 * static bool EEPROM_Application(${moduleGroupNameLowerCase}_client_transfer_event_t event)
 * {
 *     switch (event)
 *     {
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_ADDR_MATCH:
 *         if (${componentName}_TransferDirGet() == ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_DIR_WRITE)
 *         {
 *             isMemoryLocation = true;
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_RX_READY:
 *         //Read the data sent by I2C Host
 *         if (isMemoryLocation)
 *         {
 *             memoryLocation = ${componentName}_ReadByte();
 *             isMemoryLocation = false;
 *             break;
 *         }
 *         else
 *         {
 *             CLIENT_EEPROM[memoryLocation++] = ${componentName}_ReadByte();
 *             if (memoryLocation >= I2C_EEPROM_SIZE)
 *             {
 *                 memoryLocation = 0x00;
 *             }
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_TX_READY:
 *         // Provide the EEPROM data requested by the I2C Host
 *         ${componentName}_WriteByte(CLIENT_EEPROM[memoryLocation++]);
 *         if (memoryLocation >= I2C_EEPROM_SIZE)
 *         {
 *             memoryLocation = 0x00;
 *         }
 *         break;
 * 
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_STOP_BIT_RECEIVED:
 *         memoryLocation = 0x00;
 *         break;
 *     case ${moduleGroupNameUpperCase}_CLIENT_TRANSFER_EVENT_ERROR:
 *         memoryLocation = 0x00;
 *         ${moduleGroupNameLowerCase}_client_error_t errorState = ${componentName}_ErrorGet();
 *         if(errorState == ${moduleGroupNameUpperCase}_CLIENT_ERROR_BUS_ERROR)
 *         {
 *             // Bus Collision Error Handling
 *         }
 *         else if(errorState == ${moduleGroupNameUpperCase}_CLIENT_ERROR_COLLISION)
 *         {
 *             // Write Collision Error Handling
 *         }
 *         break;
 * 
 *     default:
 *         break;
 *     }
 *     return true;
 * }
 * @endcode
*/

#endif /* ${moduleNameUpperCase}_CLIENT_INTERRUPT_H */
