/**
 * \file
 * ${moduleNameLowerCase}.c
 *
 * \ingroup
 * ${componentNameLowerCase}_host
 *
 * \brief
 *  This file contains the driver code for ${moduleNameUpperCase} module.
 *
 * \version
 * I2C Host Driver Version ${hostDriverVersion}
 */

${disclaimer}

#include "../${header.name}"
#include <stdbool.h>
#include <stdlib.h>

static void ${componentNameUpperCase}_Close(void);
static void ${componentNameUpperCase}_ReadStart(void);
static void ${componentNameUpperCase}_WriteStart(void);
static void ${componentNameUpperCase}_EventHandler(void);
static void ${componentNameUpperCase}_ErrorEventHandler(void);
static void ${componentNameUpperCase}_DefaultCallback(void);

/* I2C Internal API's */
static uint8_t ${componentName}_GetRxData(void);
static void ${componentName}_SendTxData(uint8_t data);
static void ${componentName}_SendTxAddr(uint8_t data);
static inline void ${componentName}_ClearInterruptFlag(void);
static inline void ${componentName}_ResetBus(void);
static inline void ${componentName}_EnableRestart(void);
static inline void ${componentName}_DisableRestart(void);
static inline void ${componentName}_SendStop(void);
static bool ${componentName}_IsNack(void);
static bool ${componentName}_IsBusError(void);
static bool ${componentName}_IsData(void);
static bool ${componentName}_IsAddr(void);
static inline void ${componentName}_ClearErrorFlags(void);
static inline void ${componentName}_EnableInterrupts(void);
static inline void ${componentName}_DisableInterrupts(void);
static inline void ${componentName}_ClearInterrupts(void);
static inline void ${componentName}_HostSendAck(void);
static inline void ${componentName}_HostSendNack(void);
static bool ${componentName}_IsArbitrationlostOverride(void);

/* Host */
/* ${componentName} interfaces */
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_IDLE(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_SEND_RD_ADDR(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_SEND_WR_ADDR(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_TX(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_RX(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_NACK(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_ERROR(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_STOP(void);
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_RESET(void);

/**
  Section: Driver Interface
*/
const ${moduleGroupNameLowerCase}_host_interface_t ${componentNameLowerCase}_host_interface = {
  .Initialize = ${componentName}_Initialize,
  .Deinitialize = ${componentName}_Deinitialize,
  .Write = ${componentName}_Write,
  .Read = ${componentName}_Read,
  .WriteRead = ${componentName}_WriteRead,
  .TransferSetup = NULL,
  .ErrorGet = ${componentName}_ErrorGet,
  .IsBusy = ${componentName}_IsBusy,
  .CallbackRegister = ${componentName}_CallbackRegister,
  .Tasks = ${componentName}_Tasks
};

/**
 Section: Private Variable Definitions
 */

static void (*${componentNameUpperCase}_Callback)(void) = NULL;
volatile ${moduleGroupNameLowerCase}_event_status_t ${componentNameUpperCase}_Status = {0};

typedef ${moduleGroupNameLowerCase}_event_states_t (*${componentNameLowerCase}eventHandler)(void);
const ${componentNameLowerCase}eventHandler ${componentNameLowerCase}_eventTable[] = {
    ${moduleGroupNameUpperCase}_EVENT_IDLE,
    ${moduleGroupNameUpperCase}_EVENT_SEND_RD_ADDR,
    ${moduleGroupNameUpperCase}_EVENT_SEND_WR_ADDR,
    ${moduleGroupNameUpperCase}_EVENT_TX,
    ${moduleGroupNameUpperCase}_EVENT_RX,
    ${moduleGroupNameUpperCase}_EVENT_NACK,
    ${moduleGroupNameUpperCase}_EVENT_ERROR,
    ${moduleGroupNameUpperCase}_EVENT_STOP,
    ${moduleGroupNameUpperCase}_EVENT_RESET
};

void ${api.Initialize.name}(void)
{
    <#if CTRLA??>
    //${CTRLA.comment}
    ${moduleNameUpperCase}.${CTRLA.name} = ${CTRLA.valueAsHex};
    
    </#if>
    <#if DBGCTRL??>
    //Debug Run
    ${moduleNameUpperCase}.${DBGCTRL.name} = ${DBGCTRL.valueAsHex};
    
    </#if>
    <#if MBAUD??>
    //Host Baud Rate Control
    ${moduleNameUpperCase}.${MBAUD.name} = (uint8_t)${componentNameUpperCase}_BAUD(${SCL_Frequency}, ${Trise});
    
    </#if>
    <#if MCTRLA??>
    //${MCTRLA.comment}
    ${moduleNameUpperCase}.${MCTRLA.name} = ${MCTRLA.valueAsHex};
    
    </#if>
    <#if MSTATUS??>
    //${MSTATUS.comment}
    ${moduleNameUpperCase}.${MSTATUS.name} = ${MSTATUS.valueAsHex};
    
    </#if>
    <#if MADDR??>
    //Host Address
    ${moduleNameUpperCase}.${MADDR.name} = ${MADDR.valueAsHex};
    
    </#if>
    <#if MCTRLB??>
    //${MCTRLB.comment}
    ${moduleNameUpperCase}.${MCTRLB.name} = ${MCTRLB.valueAsHex};
    
    </#if>
    <#if MDATA??>
    //Host Data
    ${moduleNameUpperCase}.${MDATA.name} = ${MDATA.valueAsHex};
    
    </#if>

    ${moduleNameUpperCase}.MCTRLB |= TWI_FLUSH_bm; 
    ${moduleNameUpperCase}.MSTATUS |= TWI_BUSSTATE_IDLE_gc;

    ${componentName}_CallbackRegister(${componentNameUpperCase}_DefaultCallback);
}

void ${api.Deinitialize.name}(void)
{
    <#if CTRLA??>
    //${CTRLA.comment}
    ${moduleNameUpperCase}.${CTRLA.name} = 0x00;
    
    </#if>
    <#if DBGCTRL??>
    //Debug Run
    ${moduleNameUpperCase}.${DBGCTRL.name} = 0x00;
    
    </#if>
    <#if MBAUD??>
    //Host Baud Rate Control
    ${moduleNameUpperCase}.${MBAUD.name} = (uint8_t)${componentNameUpperCase}_BAUD(${SCL_Frequency}, ${Trise});
    
    </#if>
    <#if MCTRLA??>
    //${MCTRLA.comment}
    ${moduleNameUpperCase}.${MCTRLA.name} = 0x00;
    
    </#if>
    <#if MSTATUS??>
    //${MSTATUS.comment}
    ${moduleNameUpperCase}.${MSTATUS.name} = 0x00;
    
    </#if>
    <#if MADDR??>
    //Host Address
    ${moduleNameUpperCase}.${MADDR.name} = 0x00;
    
    </#if>
    <#if MCTRLB??>
    //${MCTRLB.comment}
    ${moduleNameUpperCase}.${MCTRLB.name} = 0x00;
    
    </#if>
    <#if MDATA??>
    //Host Data
    ${moduleNameUpperCase}.${MDATA.name} = 0x00;

    ${moduleNameUpperCase}.MCTRLB |= TWI_FLUSH_bm; 
    ${moduleNameUpperCase}.MSTATUS |= TWI_BUSSTATE_IDLE_gc;

    ${componentName}_CallbackRegister(${componentNameUpperCase}_DefaultCallback);
    
    </#if>
}

bool ${componentName}_Write(uint16_t address, uint8_t *data, size_t dataLength)
{
    bool retStatus = false;
    if (!${componentName}_IsBusy())
    {
        ${componentNameUpperCase}_Status.busy = true;
        ${componentNameUpperCase}_Status.address = address;
        ${componentNameUpperCase}_Status.switchToRead = false;
        ${componentNameUpperCase}_Status.writePtr = data;
        ${componentNameUpperCase}_Status.writeLength = dataLength;
        ${componentNameUpperCase}_Status.readPtr = NULL;
        ${componentNameUpperCase}_Status.readLength = 0;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_NONE;
        ${componentNameUpperCase}_WriteStart();
        retStatus = true;
    }
    return retStatus;
}

bool ${componentName}_Read(uint16_t address, uint8_t *data, size_t dataLength)
{
    bool retStatus = false;
    if (!${componentName}_IsBusy())
    {
        ${componentNameUpperCase}_Status.busy = true;
        ${componentNameUpperCase}_Status.address = address;
        ${componentNameUpperCase}_Status.switchToRead = false;
        ${componentNameUpperCase}_Status.readPtr = data;
        ${componentNameUpperCase}_Status.readLength = dataLength;
        ${componentNameUpperCase}_Status.writePtr = NULL;
        ${componentNameUpperCase}_Status.writeLength = 0;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_NONE;
        ${componentNameUpperCase}_ReadStart();
        retStatus = true;
    }
    return retStatus;
}

bool ${componentName}_WriteRead(uint16_t address, uint8_t *writeData, size_t writeLength, uint8_t *readData, size_t readLength)
{
    bool retStatus = false;
    if (!${componentName}_IsBusy())
    {
        ${componentNameUpperCase}_Status.busy = true;
        ${componentNameUpperCase}_Status.address = address;
        ${componentNameUpperCase}_Status.switchToRead = true;
        ${componentNameUpperCase}_Status.writePtr = writeData;
        ${componentNameUpperCase}_Status.writeLength = writeLength;
        ${componentNameUpperCase}_Status.readPtr = readData;
        ${componentNameUpperCase}_Status.readLength = readLength;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_NONE;
        ${componentNameUpperCase}_WriteStart();
        retStatus = true;
    }
    return retStatus;
}

${moduleGroupNameLowerCase}_host_error_t ${componentName}_ErrorGet(void)
{
    ${moduleGroupNameLowerCase}_host_error_t retErrorState = ${componentNameUpperCase}_Status.errorState;
    ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_NONE;
    return retErrorState;
}

bool ${componentName}_IsBusy(void)
{
    return ${componentNameUpperCase}_Status.busy || !(${moduleNameUpperCase}.MSTATUS & TWI_BUSSTATE_IDLE_gc);
}

void ${componentName}_CallbackRegister(void (*callbackHandler)(void))
{
    if (callbackHandler != NULL)
    {
        ${componentNameUpperCase}_Callback = callbackHandler;
    }
}

void ${componentName}_Tasks(void)
{
    bool retStatus = ${componentName}_IsBusy();
    if (retStatus)
    {
        if(((${moduleNameUpperCase}.MSTATUS & TWI_RXACK_bm) && (${moduleNameUpperCase}.MSTATUS & TWI_WIF_bm) && (!(${moduleNameUpperCase}.MSTATUS & TWI_BUSERR_bm)) && (!(${moduleNameUpperCase}.MSTATUS & TWI_ARBLOST_bm))) || (${moduleNameUpperCase}.MSTATUS & TWI_BUSERR_bm) || (${moduleNameUpperCase}.MSTATUS & TWI_ARBLOST_bm))
        {
            ${componentNameUpperCase}_ErrorEventHandler();
        }
        if ((${moduleNameUpperCase}.MSTATUS & TWI_RIF_bm) || (${moduleNameUpperCase}.MSTATUS & TWI_WIF_bm))
        {
            if ((${moduleNameUpperCase}.MSTATUS & TWI_RXACK_bm) || (${moduleNameUpperCase}.MSTATUS & TWI_BUSERR_bm) || (${moduleNameUpperCase}.MSTATUS & TWI_ARBLOST_bm))
            {
                ${componentNameUpperCase}_ErrorEventHandler();
            }
            else
            {
                ${componentNameUpperCase}_EventHandler();
            }
        }
    }
}

/**
 Section: Private Interfaces
 */
static void ${componentNameUpperCase}_ReadStart(void)
{
    ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_EVENT_SEND_RD_ADDR();
}

static void ${componentNameUpperCase}_WriteStart(void)
{
    ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_EVENT_SEND_WR_ADDR();
}

static void ${componentNameUpperCase}_Close(void)
{
    ${componentNameUpperCase}_Status.busy = false;
    ${componentNameUpperCase}_Status.address = 0xFF;
    ${componentNameUpperCase}_Status.writePtr = NULL;
    ${componentNameUpperCase}_Status.readPtr = NULL;
    ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_STATE_IDLE;
    ${componentName}_ClearInterrupts();
    ${componentName}_DisableInterrupts();
    ${moduleNameUpperCase}.MSTATUS |= TWI_BUSSTATE_IDLE_gc;
}

static void ${componentNameUpperCase}_EventHandler(void)
{
    ${componentNameUpperCase}_Status.state = ${componentNameLowerCase}_eventTable[${componentNameUpperCase}_Status.state]();
}

static void ${componentNameUpperCase}_ErrorEventHandler(void)
{
    if (${componentName}_IsBusError())
    {
        ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_STATE_ERROR;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_BUS_COLLISION;
        ${moduleNameUpperCase}.MSTATUS |= TWI_BUSERR_bm;
    }
    else if (${componentName}_IsAddr() && ${componentName}_IsNack())
    {
        ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_STATE_NACK;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_ADDR_NACK;
        ${moduleNameUpperCase}.MSTATUS |= TWI_RXACK_bm;
    }
    else if (${componentName}_IsData() && ${componentName}_IsNack())
    {
        ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_STATE_NACK;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_DATA_NACK;
        ${moduleNameUpperCase}.MSTATUS |= TWI_RXACK_bm;
    }
    else if(${componentName}_IsArbitrationlostOverride())
    {
        ${componentNameUpperCase}_Status.state = ${moduleGroupNameUpperCase}_STATE_ERROR;
        ${componentNameUpperCase}_Status.errorState = ${moduleGroupNameUpperCase}_ERROR_BUS_COLLISION;
        ${moduleNameUpperCase}.MSTATUS |= TWI_ARBLOST_bm;
    }
    ${componentNameUpperCase}_Status.state = ${componentNameLowerCase}_eventTable[${componentNameUpperCase}_Status.state]();
    if(${componentNameUpperCase}_Status.errorState != ${moduleGroupNameUpperCase}_ERROR_NONE)
    {
        ${componentNameUpperCase}_Callback();
    }
}

static void ${componentNameUpperCase}_DefaultCallback(void)
{
    // Default Callback for Error Indication
}

/* ${componentName} Event interfaces */
static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_IDLE(void)
{
    ${componentNameUpperCase}_Status.busy = false;
    ${moduleNameUpperCase}.MSTATUS |= TWI_BUSSTATE_IDLE_gc;
    return ${moduleGroupNameUpperCase}_STATE_RESET;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_SEND_RD_ADDR(void)
{
    ${componentName}_SendTxAddr((uint8_t) (${componentNameUpperCase}_Status.address << 1 | 1));
    return ${moduleGroupNameUpperCase}_STATE_RX;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_SEND_WR_ADDR(void)
{
    ${componentName}_SendTxAddr((uint8_t) (${componentNameUpperCase}_Status.address << 1));
    return ${moduleGroupNameUpperCase}_STATE_TX;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_TX(void)
{
    ${moduleGroupNameLowerCase}_event_states_t retEventState = ${moduleGroupNameUpperCase}_STATE_TX;

        if (${componentNameUpperCase}_Status.writeLength)
        {
            ${componentNameUpperCase}_Status.writeLength--;
            ${componentName}_SendTxData(*${componentNameUpperCase}_Status.writePtr++);
            retEventState = ${moduleGroupNameUpperCase}_STATE_TX;
        }
        else
        {
            if (${componentNameUpperCase}_Status.switchToRead)
            {
                ${componentNameUpperCase}_Status.switchToRead = false;
                retEventState = ${moduleGroupNameUpperCase}_STATE_SEND_RD_ADDR;
            }
            else
            {
                retEventState = ${moduleGroupNameUpperCase}_EVENT_STOP();
            }
        }

    return retEventState;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_RX(void)
{
    ${moduleGroupNameLowerCase}_event_states_t retEventState = ${moduleGroupNameUpperCase}_STATE_RX;

    if (${componentNameUpperCase}_Status.readLength == 1)
    {
        // Next byte will be last to be received, setup NACK
        ${componentName}_HostSendNack();
    }
    else
    {
        // More bytes to receive, setup ACK
        ${componentName}_HostSendAck();
    }

    if (--${componentNameUpperCase}_Status.readLength)
    {
        *${componentNameUpperCase}_Status.readPtr++ = ${componentName}_GetRxData();
        //Execute Acknowledge Action followed by a byte read operation
        ${moduleNameUpperCase}.MCTRLB |= TWI_MCMD_RECVTRANS_gc;
        retEventState = ${moduleGroupNameUpperCase}_STATE_RX;
    }
    else
    {
        *${componentNameUpperCase}_Status.readPtr++ = ${componentName}_GetRxData();
        ${componentName}_HostSendNack();
        retEventState = ${moduleGroupNameUpperCase}_EVENT_STOP();
    }

    return retEventState;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_NACK(void)
{
    ${moduleGroupNameLowerCase}_event_states_t retEventState = ${moduleGroupNameUpperCase}_STATE_NACK;
    retEventState = ${moduleGroupNameUpperCase}_EVENT_STOP();
    return retEventState;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_ERROR(void)
{
    // Clear bus collision status flag
    ${moduleGroupNameLowerCase}_event_states_t retEventState = ${moduleGroupNameUpperCase}_STATE_ERROR;
    ${componentName}_ClearInterrupts();
    retEventState = ${moduleGroupNameUpperCase}_EVENT_RESET();
    return retEventState;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_STOP(void)
{
    ${componentName}_SendStop();
    ${componentNameUpperCase}_Close();
    return ${moduleGroupNameUpperCase}_STATE_IDLE;
}

static ${moduleGroupNameLowerCase}_event_states_t ${moduleGroupNameUpperCase}_EVENT_RESET(void)
{
    ${componentName}_ResetBus();
    ${componentNameUpperCase}_Status.busy = false;
    return ${moduleGroupNameUpperCase}_STATE_IDLE;
}

/**
 Section: Register Level Interfaces
 */
static uint8_t ${componentName}_GetRxData(void)
{
    return ${moduleNameUpperCase}.MDATA;
}

static void ${componentName}_SendTxData(uint8_t data)
{
    ${moduleNameUpperCase}.MDATA = data;
}

static void ${componentName}_SendTxAddr(uint8_t data)
{
    ${moduleNameUpperCase}.MADDR = data;
}

static inline void ${componentName}_ResetBus(void)
{
    //Set Clear Buffer Flag
    ${moduleNameUpperCase}.MCTRLA &= ~(1 << TWI_ENABLE_bp);
    ${moduleNameUpperCase}.MCTRLA |= 1 << TWI_ENABLE_bp;
    ${moduleNameUpperCase}.MSTATUS |= TWI_BUSSTATE_IDLE_gc;
}

static void ${componentName}_ClearInterruptFlag(void)
{
    if(${moduleNameUpperCase}.MSTATUS & TWI_RIF_bm)
    {
        ${moduleNameUpperCase}.MSTATUS|= TWI_RIF_bm;
    }
    else if(${moduleNameUpperCase}.MSTATUS & TWI_WIF_bm)
    {
        ${moduleNameUpperCase}.MSTATUS|= TWI_WIF_bm;
    }
}

static inline void ${componentName}_EnableRestart(void)
{
    //Enable ${componentName} Restart
    ${moduleNameUpperCase}.MCTRLB |= TWI_MCMD_REPSTART_gc;
}

static inline void ${componentName}_DisableRestart(void)
{
    //Disable ${componentName} Restart
    ${moduleNameUpperCase}.MCTRLB &= ~TWI_MCMD_REPSTART_gc;
}

static inline void ${componentName}_SendStop(void)
{
    //Clear Start Bit
    ${moduleNameUpperCase}.MCTRLB |= TWI_MCMD_STOP_gc;
}

static bool ${componentName}_IsNack(void)
{
    return ${moduleNameUpperCase}.MSTATUS & TWI_RXACK_bm;
}

static bool ${componentName}_IsBusError(void)
{
    return ${moduleNameUpperCase}.MSTATUS & TWI_BUSERR_bm;
}

static bool ${componentName}_IsData(void)
{
    return ${moduleNameUpperCase}.MDATA;
}

static bool ${componentName}_IsAddr(void)
{
    return ${moduleNameUpperCase}.MADDR;
}

static bool ${componentName}_IsArbitrationlostOverride(void)
{
    return ${moduleNameUpperCase}.MSTATUS & TWI_ARBLOST_bm;
}

static inline void ${componentName}_ClearErrorFlags(void)
{
   ${moduleNameUpperCase}.MSTATUS |= TWI_RXACK_bm;
   ${moduleNameUpperCase}.MSTATUS |= TWI_BUSERR_bm;
   ${moduleNameUpperCase}.MSTATUS |= TWI_ARBLOST_bm;
}

static inline void ${componentName}_HostSendAck(void)
{
    ${moduleNameUpperCase}.MCTRLB &= ~(1 << TWI_ACKACT_bp);
}

static inline void ${componentName}_HostSendNack(void)
{
    ${moduleNameUpperCase}.MCTRLB |= TWI_ACKACT_NACK_gc;
}

static inline void ${componentName}_EnableInterrupts(void)
{
    ${moduleNameUpperCase}.MCTRLA |= (TWI_RIEN_bm | TWI_WIEN_bm);
}

static inline void ${componentName}_DisableInterrupts(void)
{
    ${moduleNameUpperCase}.MCTRLA &= ~(TWI_RIEN_bm | TWI_WIEN_bm);
}

static inline void ${componentName}_ClearInterrupts(void)
{
   ${moduleNameUpperCase}.MSTATUS |= (TWI_RIF_bm | TWI_WIF_bm);
}
